                .LF     keymap.lst
;------------------------------------------------------------------------
;
;  keymap.asm
;
;  A simple program to find out how the keyboard keys are arranged in the
;  keyboard matrix.
;
;  Author : San Bergmans
;  Date   : 2-12-2005
;
;------------------------------------------------------------------------
;
;  This program shows 2 digits. The left digit shows the input row bit
;  of the pressed key (8255 PortA is the input row). The right digit
;  shows the output column bit number of the pressed key (82155 PortC is
;  the column output).
;  When no key is pressed the display will be blank.
;
;  I used this program to examine the keyboard matrix layout. It's not
;  much use any longer now I know how the keyboard matrix of the MPF-1 is
;  connected.
;
;------------------------------------------------------------------------

                .CR     Z80             Yeah! We're programming the Z80!
                .OR     01800H          RAM starts here, so are we
                .TF     keymap.hex,HEX  Create a plain hex dump file

;------------------------------------------------------------------------
;  Constants
;------------------------------------------------------------------------

KIN             .EQ     000H            8255 Port A, the matrix input
DIGIT           .EQ     002H            8255 Port C, the matrix output

;------------------------------------------------------------------------
;  Monitor routines
;------------------------------------------------------------------------

SCAN1           .EQ     00624H          Scan display once
HEX7SG          .EQ     00678H          Convert 2 hex digits to 7-segment

;------------------------------------------------------------------------
;  The KEYMAP program itself
;  Register use:
;       A       misc
;       B       Row counter
;       C       Row index counter
;       D       Column index counter
;       E       Column output mask
;       L       Column counter
;       H       Temp row
;------------------------------------------------------------------------

KEYMAP          LD      E,11000001B     Starting output mask
                LD      D,0             Clear the column index
                LD      L,6             Set the column counter to 6

.COLUMN       ; Column scan loop
                LD      A,E             Make only current column low
                CPL
                OR      11000000B       Don't mess with these two bits!
                OUT     (DIGIT),A

                LD      B,6             Set row counter to 6
                LD      C,0             Clear the row index
                IN      A,(KIN)         See if key down in this column
                LD      H,A

.ROW          ; Row scan loop
                RR      H               Is a key down in this row?
                JR      NC,.KEY         Yes, this bit was low!
                INC     C               No! Increment Row index
                DJNZ    .ROW            Repeat for all rows!

                INC     D               No key down in this column!
                LD      A,E             Shift output mask 1 bit to left
                AND     00111111B
                RLCA
                OR      11000000B       Without altering these
                LD      E,A
                DEC     L               Decrement column counter
                JR      NZ,.COLUMN      Repeat for all columns

                LD      A,0             No key was pressed, simply clear
                LD      (DSPBFR),A       the digits from the display
                LD      (DSPBFR+1),A      buffer
                JP      .SHOW           And show display buffer

.KEY          ; A key was pressed, display it!
                LD      A,C             Get row index and move it to
                RLCA                     upper nibble of the byte
                RLCA
                RLCA
                RLCA

                OR      A,D             Combine it with the column number
                LD      HL,DSPBFR       Put the 7-segment patterns here,
                CALL    HEX7SG           which is done by this routine


.SHOW         ; Scan the display again before repeating everything
                LD      IX,DSPBFR       Load display buffer index
                CALL    SCAN1            and scan the display once
                JP      KEYMAP          Repeat entire program for ever

;------------------------------------------------------------------------
;  Display buffer
;------------------------------------------------------------------------

DSPBFR          .DA     #000H           Initially show "SBP   "
                .DA     #000H
                .DA     #000H
                .DA     #01FH
                .DA     #0A7H
                .DA     #0AEH

                .LI     OFF
